;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; whitewash.asm
;;
;; Copyright 2000 G. Adam Stanislav
;; All rights reserved
;;
;; Started: 04-Feb-2000
;; Updated: 04-Feb-2000
;;
;; This is a Photoshop plugin to whitewash an image alpha channel.
;;
;; This source code can be assembled by NASM:
;;
;;	nasm -f win32 whitewash.asm
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

%include	'../filter.inc'

global	filter.parameters, filter.prepare, filter.start
global	filter.continue, filter.finish, filter.name

extern	__imp__MessageBoxA@16

section	.drectve	info
db	"-out:rpWWash.8bf "
db	"-defaultlib:user32.lib "
db	"-entry:filterDLL "
db	"-DLL "
db	"-version:1.0 "
db	"-machine:i386 "
db	"-subsystem:windows,4.0 "
db	"-export:rpFilter "

section	.data
total	dd	0
partial	dd	0
filter.name	db	'Whitewash', 0
filter.sorry	db	'Sorry, this selection has no alpha channel.', 0
filter.author	db	"Copyright ", 169, " 2000, G. Adam Stanislav", 0Ah
	db	"All rights reserved.", 0Ah, 0Ah
	db	"http://www.redprince.net/", 0

section	.text align=4

align 4
global	_filterDLL@12
_filterDLL@12:

	mov	al, 1
	ret	12

align 4
filter.about:
	sub	eax, eax
	mov	ecx, dword [esp+8]	; address of hWnd
	push	eax
	push	dword filter.name
	push	dword filter.author
	jecxz	.@@
	mov	eax, [ecx]

.@@:
	push	eax
	call	dword [__imp__MessageBoxA@16]
	ret

align 4
calls	dd	filter.about
	dd	filter.parameters
	dd	filter.prepare
	dd	filter.start
	dd	filter.continue
	dd	filter.finish
numcalls	equ	($-calls)/4

align 4
global	rpFilter
rpFilter:

	; We receive four parameters:
	;
	;	[esp+ 4] = selector (word)
	;	[esp+ 8] = address of filterParamBlock
	;	[esp+12] = address of data
	;	[esp+16] = address of result word 

	mov	ecx, [esp+16]	; result
	jecxz	.error

	mov word [ecx], 0
	movzx	eax, word [esp+4]
	cmp	eax, numcalls
	jae	.unknown
	jmp	dword [calls+eax*4]

align 4
.unknown:
	mov	word [ecx], -30100

.error:
	ret

	; These procedures receive four parameters:
	;
	;	[esp+ 4] = selector (word) = EAX
	;	[esp+ 8] = address of filterParamBlock
	;	[esp+12] = address of data
	;	[esp+16] = address of result word, contains 0 (success)

align 4
error.return:
	mov	ecx, [esp+16]
	mov	word [ecx], filterBadParameters
	ret

align 4
filter.start:
	; Get the pointer to the parameter block
	mov	ecx, [esp+8]

	; Quit if NULL.
	jecxz	error.return

	; Tell the host to give us alpha plane for the output only
	mov	ax, [ecx+outLayerPlanes]
	mov	dx, ax
	shl	eax, 16
	mov	ax, dx
	mov	[ecx+outLoPlane], eax
	; Ask for the R, G, B planes for the input only
	mov	eax, 2 << 16
	mov	[ecx+inLoPlane], eax

	; Calculate the total number of pixels
	movzx	eax, word [ecx+filterRect+right]
	sub	ax, [ecx+filterRect+left]
	movzx	edx, word [ecx+filterRect+bottom]
	sub	dx, [ecx+filterRect+top]
	mul	edx
	mov	[total], eax

	sub	edx, edx
	mov	[partial], edx

	; Ask the host for a NUMCOLSxNUMROWS input and output data. Less if the
	; image is small.
	movzx	eax, word [ecx+filterRect+right]
	movzx	edx, word [ecx+filterRect+left]
	sub	ax, dx

	cmp	ax, NUMCOLS
	jb	.X
	mov	ax, NUMCOLS-1

.X:
	mov	[ecx+inRect+left], dx
	mov	[ecx+outRect+left], dx
	add	ax, dx
	mov	[ecx+inRect+right], ax
	mov	[ecx+outRect+right], ax

	mov	ax, [ecx+filterRect+bottom]
	mov	dx, [ecx+filterRect+top]
	sub	ax, dx

	cmp	ax, NUMROWS
	jb	.Y
	mov	ax, NUMROWS-1

.Y:
	mov	[ecx+inRect+top], dx
	mov	[ecx+outRect+top], dx
	add	ax, dx
	mov	[ecx+inRect+bottom], ax
	mov	[ecx+outRect+bottom], ax

	ret

align 4
filter.continue:

	mov	ecx, [esp+8]
	or	ecx, ecx
	je	near error.return

	; Quit if there is no alpha channel
	mov	eax, [ecx+outData]
	or	eax, eax
	je	near filter.cleanup

	; Calculate how far we are
	movzx	eax, word [ecx+outRect+right]
	sub	ax, [ecx+outRect+left]
	movzx	edx, word [ecx+outRect+bottom]
	sub	dx, [ecx+outRect+top]
	mul	edx
	add	[partial], eax

	push	ecx
	push	dword [total]
	push	dword [partial]
	call	[ecx+progressProc]
	add	esp, 8
	pop	edx

	push	esi
	push	edi

	mov	esi, [edx+inData]
	mov	edi, [edx+outData]

	movzx	ecx, word [edx+outRect+bottom]
	sub	cx, [edx+outRect+top]

	sub	eax, eax
	cld

	; ECX = number of rows

.bigloop:
	push	ecx

	mov	cx, word [edx+outRect+right]
	sub	cx, [edx+outRect+left]

	; ECX = number of columns

	push	esi
	push	edi

.filterloop:

	lodsw
	cmp	al, ah
	jae	.f1
	mov	ah, al

.f1:
	lodsb
	cmp	al, ah
	jbe	.f2
	mov	al, ah

.f2:
	not al
	stosb

	loop	.filterloop

	pop	edi
	pop	esi

	add	esi, [edx+inRowBytes]
	add	edi, [edx+outRowBytes]
	pop	ecx
	loop	.bigloop

	pop	edi
	pop	esi

	mov	ecx, edx

	; See if there are more columns to process within these rows.
	movzx	eax, word [ecx+outRect+right]
	cmp	ax, [ecx+filterRect+right]
	jae	.rows

.left:
	mov	[ecx+inRect+left], ax
	mov	[ecx+outRect+left], ax
	add	ax, NUMCOLS-1
	cmp	ax, [ecx+filterRect+right]
	jbe	.right
	mov	ax, [ecx+filterRect+right]

.right:
	mov	[ecx+inRect+right], ax
	mov	[ecx+outRect+right], ax

	ret

.rows:
	; See if there are any unprocessed rows
	movzx	eax, word [ecx+inRect+bottom]
	cmp	ax, [ecx+filterRect+bottom]
	jae	filter.cleanup	; all done
	mov	[ecx+inRect+top], ax
	mov	[ecx+outRect+top], ax
	add	ax, NUMROWS-1
	cmp	ax, [ecx+filterRect+bottom]
	jbe	.bottom
	mov	ax, [ecx+filterRect+bottom]

.bottom:
	mov	[ecx+inRect+bottom], ax
	mov	[ecx+outRect+bottom], ax

	movzx	eax, word [ecx+filterRect+left]
	jmp	.left


align 4
filter.cleanup:
	sub	eax, eax
	mov	[ecx+inRect], eax
	mov	[ecx+inRect+4], eax
	mov	[ecx+outRect], eax
	mov	[ecx+outRect+4], eax
	mov	[ecx+inLoPlane], eax
	mov	[ecx+outLoPlane], eax

.done:
	ret

align 4
filter.prepare:

	mov	ecx, [esp+8]
	or	ecx, ecx
	jz	near error.return

	mov	ax, word [ecx+outTransparencyMask]
	or	ax, ax
	je	sorry

	mov	eax, NUMROWS*NUMCOLS*4+NUMROWS*3
	cmp	eax, [ecx+maxSpace]
	ja	near error.return
	mov	[ecx+maxSpace], eax
	sub	eax, eax
	mov	[ecx+bufferSpace], eax

	ret

align 4
filter.finish:
	mov	ecx, [esp+12]
	jecxz	filter.parameters
	sub	eax, eax
	mov	[ecx], eax

	; FALL THROUGH

filter.parameters:
	; No parameters required. Just return.

	ret

align 4
sorry:
	mov	ecx, [esp+16]
	sub	eax, eax
	push	eax
	push	dword filter.name
	push	dword filter.sorry
	push	eax
	inc	al
	mov	[ecx], al
	call	dword [__imp__MessageBoxA@16]
	ret

align 4
